// ad10led3.c - measure input voltage from pot divider using ADC10
// Readings taken at intervals set by watchdog timer, action in ISR
// ADC10 default references (VSS, VCC), internal clock, polling loop
// Home-made analog demo board for MSP430F20xx:
//   10k on P1.5, 20k variable on P1.6, junction to P1.1 with 10nF cap
//   LED active high on P2.6
// Default DCO, no crystal, no ACLK, power from JTAG
// J H Davies, 2007-08-30; IAR Kickstart version 3.42A
//----------------------------------------------------------------------
#include <io430x20x2.h>				// Header file for this device
#include <intrinsics.h>				// Intrinsic functions
#include <stdint.h>					// Standard integer types

#define	LED		P2OUT_bit.P2OUT_6	// Output bit for LED

void main (void)
{
	BCSCTL3 = LFXT1S_2;				// Select ACLK from VLO (no crystal)
// Watchdog as interval timer, clear, ACLK / 32768
	WDTCTL = WDTPW | WDTTMSEL | WDTCNTCL | WDTSSEL;
	IE1_bit.WDTIE = 1;				// WDT interval timer interrupts
	P2SEL = 0;						// Digital i/o rather than crystal
	P2OUT = 0;
	P2DIR = BIT6|BIT7;				// Unused outputs driven low
	P1OUT = BIT5;					// VCC to 10k fixed resistor
	P1DIR = BIT3|BIT5|BIT6|BIT7;	// Unused outputs driven low
// VCC and VSS refs, sample for 16 cycs, int ref off, ADC off, no ints
	ADC10CTL0 = SREF_0 | ADC10SHT_2;
// Input channel 1, trigger using ADC10SC bit, 2s complement,
//   no clock division, internal osc, single channel single conversions
	ADC10CTL1 = INCH_1|SHS_0|ADC10DF|ADC10DIV_0|ADC10SSEL_0|CONSEQ_0;
	ADC10AE0 = BIT1;				// Enable analog input on channel 1
	for (;;) {						// Loop forever taking measurements
		__low_power_mode_3();		// Sleep between measurements
	}
}
//----------------------------------------------------------------------
// Interrupt service routine for watchdog: take new reading, update LED
//----------------------------------------------------------------------
#pragma vector = WDT_VECTOR
__interrupt void WDT_ISR (void)		// Acknowledged automatically
{
	LED ^= 1;						// Flash LED to show activity
	ADC10CTL0_bit.ADC10ON = 1;		// Turn on ADC
	ADC10CTL0 |= (ENC|ADC10SC);		// Enable ADC10, start conversion
	while (ADC10CTL1_bit.ADC10BUSY == 1) {
	}								// Loop while conversion takes place
	ADC10CTL0_bit.ENC = 0;			// Disable conversions
	ADC10CTL0_bit.ADC10ON = 0;		// Turn off ADC
	if (((int16_t) ADC10MEM) > 0) {	// Is output positive?
		LED = 1;					// Yes: Turn LED on
	} else {
		LED = 0;					// No: Turn LED off
	}
}
