// butstmc2.c - state machine to toggle LED1 when button B1 pressed
// Olimex 1121STK board, 32KHz ACLK
// LED1 on P2.3, LED2 on P2.4, button B1 on P2.1, all active low
// Timer_A in up mode from ACLK to give 2Hz tick for state machine
// J H Davies, 2007-12-31; IAR Kickstart version 4.09A
//----------------------------------------------------------------------
#include <io430x11x1.h>					// Specific device
#include <intrinsics.h>

// Pins for LEDs and button
#define	LED		P2OUT_bit.P2OUT_3		// Controlled LED
#define	Monitor	P2OUT_bit.P2OUT_4		// Flash to show state mc active
#define	Button	P2IN_bit.P2IN_1			// Button to control LED
#define	ON		0						// Active low LED and button
#define	OFF		1

void main (void)
{
	WDTCTL = WDTPW | WDTHOLD;			// Disable watchdog
	LED = OFF;							// LEDs initially off
	Monitor = OFF;
	P2DIR = BIT3 | BIT4;				// Set pins with LEDs to output
	TACCR0 = 0x3FFF;					// Upper limit of count for TAR
	TACCTL0 = CCIE;						// Enable interrupts on Compare 0
// Clock from ACLK, no clock division, up mode, clear TAR
	TACTL = TASSEL_1 | ID_0 | MC_1 | TACLR;	// Set up and start Timer A
	for (;;) {							// Loop forever
		__low_power_mode_3();			// LPM3 with interrupts
	}
}
//----------------------------------------------------------------------
// Interrupt service routine for Timer A channel 0 to run state machine
//----------------------------------------------------------------------
#pragma vector = TIMERA0_VECTOR
__interrupt void TA0_ISR (void)			// Acknowledged automatically
{
	static enum {
		LedOffButUp,
		LedOnButDown,
		LedOnButUp,
		LedOffButDown
	} LedState = LedOffButUp;			// State variable, initialized

	Monitor = ON;
	switch (LedState) {
	case LedOffButUp:					// LED currently off, button up
		if (Button == ON) {				// Button pressed?
			LED = ON;					// Yes: light LED
			LedState = LedOnButDown;	// Change state
		}
		break;
	case LedOnButDown:					// LED currently on, button down
		if (Button == OFF) {			// Button released?
			LedState = LedOnButUp;		// Yes: change state
		}
		break;
	case LedOnButUp:					// LED currently on, button up
		if (Button == ON) {				// Button pressed?
			LED = OFF;					// Yes: extinguish LED
			LedState = LedOffButDown;	// Change state
		}
		break;
	case LedOffButDown:					// LED currently off, button down
		if (Button == OFF) {			// Button released?
			LedState = LedOffButUp;		// Yes: change state
		}
		break;
	default:							// Should never happen!
		LedState = LedOffButUp;			// Reset to initial state
		break;
	}
	Monitor = OFF;
}
