// ezpwm2.c - as ezpwm1.c but with a fix for duty cycle = 0
// J H Davies, 2007-07-18; IAR Kickstart version 3.42A
//----------------------------------------------------------------------
#include <io430x20x3.h>			// Specific device
#include <intrinsics.h>			// Intrinsic functions
#include <stdint.h>				// Integers of defined sizes
//----------------------------------------------------------------------
void main (void)
{
	WDTCTL = WDTPW | WDTHOLD;	// Stop watchdog timer
	BCSCTL3 = LFXT1S_2;			// ACLK from VLO
// Select fastest MCLK (16MHz) to ensure time for processing interrupts
	BCSCTL1 = CALBC1_16MHZ;		// Set range for calibrated 16MHz
	DCOCTL = CALDCO_16MHZ;		// Set DCO step and modulation
// Set up Timer_A for PWM on active high LED, channel 1
	TACCR0 = 110;				// Roughly 110Hz from 12kHz timer clock
	TACCR1 = 0x0000;			// Duty cycle D = 0 initially
	TACCTL0 = CCIE;				// Interrupts on compare to update PWM
	TACCTL1 = OUTMOD_7 | CCIE;	// Reset/set (positive PWM), interrupts
// Configure ports 1 and 2 with pull resistors on unused pins
	P1OUT = 0;					// Preclear output buffer
	P1DIR = BIT0;				// Set P1.0 to output, others input
	P1REN = 0xFF & ~BIT0;		// Pull resistors on P1[7:1]
	P2SEL = 0;					// Digital i/o rather than crystal
	P2REN = BIT6 | BIT7;		// Pull resistors on pins that exist
// Start timer from ACLK, no division, Up mode, clear, TAIFG interrupts
	TACTL = TASSEL_1 | ID_0 | MC_1 | TACLR | TAIE;
	for (;;) {					// Loop forever
		__low_power_mode_3();	// Enter LPM3 between interrupts
	}
}
//----------------------------------------------------------------------
// Interrupt service routine for CCIFG1 and TAIFG; share vector
//----------------------------------------------------------------------
#pragma vector = TIMERA1_VECTOR
__interrupt void TIMERA1_ISR (void)	// ISR for CCIFG1 and TAIFG
{
	switch (__even_in_range(TAIV, 10)) {
	case 0:						// No interrupt pending
		break;					// No action
	case TAIV_CCIFG1:			// Vector 2: CCIFG1
		P1OUT_bit.P1OUT_0 = 0;	// End of duty cycle: Turn off LED
		break;
	case TAIV_TAIFG:			// Vector A: TAIFG, last value possible
		if (TACCR1 > 0) {		// Turn on output only if duty cycle > 0
			P1OUT_bit.P1OUT_0 = 1;	// Start of PWM cycle: Turn on LED
		}
		break;
	default:					// Should not be possible
		for (;;) {				// Disaster! Loop here for ever
		}
	}
}
//----------------------------------------------------------------------
// Interrupt service routine for TACCR0.CCIFG
//----------------------------------------------------------------------
#pragma vector = TIMERA0_VECTOR
__interrupt void TIMERA0_ISR (void)	// Flag cleared automatically
{
	static enum {up, down} direction = up;

	if (direction == up) {		// Increasing duty cycle?
		++TACCR1;				// Yes: increase TACCR1
		if (TACCR1 > TACCR0) {	// Hit 100% limit?
			direction = down;	// Yes: decrease duty cycle next
		}
	} else {
		--TACCR1;				// Decrease duty cycle
		if (TACCR1 == 0) {		// Hit 0% limit?
			direction = up;		// Yes: increase duty cycle next
		}
	}
}
