// flashled3.c - toggles LEDs with period of about 1s
// Software delay in subroutine, pass delay in units of 0.1s
// Olimex 1121STK, LED1,2 active low on P2.3,4
// J H Davies, 2006-06-03; IAR Kickstart version 3.41A
//----------------------------------------------------------------------
#include <msp430x11x1.h>		// Specific device
#include <stdint.h>				// For uint16_t

// Pins for LEDs
#define	LED1	BIT3
#define	LED2	BIT4

void DelayTenths (unsigned int Tenths);		// Delay in units of 0.1s

void main (void)
{
	WDTCTL = WDTPW | WDTHOLD;	// Stop watchdog timer
	P2OUT = ~LED1;				// Preload LED1 on, LED2 off
	P2DIR = LED1|LED2;			// Set pins with LED1,2 to output
	for (;;) {					// Loop forever
		DelayTenths(5);			// Delay 0.5s
		P2OUT ^= LED1|LED2;		// Toggle LEDs
	}
}
//----------------------------------------------------------------------
void DelayTenths (unsigned int Tenths)
{
	volatile uint16_t LoopCtr;	// Unsigned 16 bits, volatile!

	if (Tenths > 5) {			// Is requested delay too large?
		LoopCtr = 0xFFFF;		// Yes: maximum delay with single loop
	} else {
		LoopCtr = 10000*Tenths;	// No: scale loop roughly to suit
	}
	while (LoopCtr-- > 0) {		// Not >=0, which must always be true!
	}							// Empty delay loop
}
