// flashled31.c - toggles LEDs with period of about 2s
// Software delay in subroutine, pass (long) delay in units of 0.1s
// Olimex 1121STK, LED1,2 active low on P2.3,4
// J H Davies, 2008-03-19; IAR Kickstart version 3.41A
//----------------------------------------------------------------------
#include <msp430x11x1.h>		// Specific device
#include <stdint.h>				// For uint16_t

// Pins for LEDs
#define	LED1	BIT3
#define	LED2	BIT4

void DelayTenths (unsigned int Tenths);		// Delay in units of 0.1s

void main (void)
{
	WDTCTL = WDTPW | WDTHOLD;	// Stop watchdog timer
	P2OUT = ~LED1;				// Preload LED1 on, LED2 off
	P2DIR = LED1|LED2;			// Set pins with LED1,2 to output
	for (;;) {					// Loop forever
		DelayTenths(10);		// Delay 1.0s (boring!)
		P2OUT ^= LED1|LED2;		// Toggle LEDs
	}
}
//----------------------------------------------------------------------
#define DELAYLOOPS	10000		// Loops for 0.1s delay

void DelayTenths (unsigned int Tenths)
{
	volatile uint16_t LoopCtr;	// Unsigned 16 bits, volatile!
	
	while (Tenths-- > 0) {
		LoopCtr = DELAYLOOPS;	// Initialize loop for 0.1s delay
		while (LoopCtr-- > 0) {	// Not >=0, which must always be true!
		}						// Empty delay loop
	}
}
