// keypad3.c - return value from telephone keypad using Wilsmhurst method
// Minimal test harness in main()
// MSPF2001 with keypad connected to port 1; needs internal pullups
// J H Davies, 2007-04-28; IAR Kickstart version 3.42A
//
// P1.7 "ignored"
// Keypad arranged like this on telephone keypad with * = 10 and # = 11:
//         P1.6 P1.5 P1.4
//          X1   X2   X3
// P1.0 Y1:  1    2    3
// P1.1 Y2:  4    5    6
// P1.2 Y3:  7    8    9
// P1.3 Y4:  *    0    #
//----------------------------------------------------------------------
#include <io430x20x1.h>				// Specific device
#include <intrinsics.h>				// Intrinsic functions
#include <stdint.h>					// Standard integer types

#define	NOKEYPRESSED	-1
#define	KEYERROR		-2

#define	NCOLS			3
#define	NROWS			4
#define	ROWSMASK		0x0F
#define	COLSMASK		0x70

// Values associated with keys located by index
const int8_t keyValue[4][3] =
	{{1, 2, 3}, {4, 5, 6}, {7, 8, 9}, {10, 0, 11}};

// Function prototypes
void KeypadInit (void);
int16_t	KeypadRead (void);

void main (void)
{
	uint16_t keypadValue;
	WDTCTL = WDTPW | WDTHOLD;		// Stop watchdog timer
	KeypadInit();					// Set default state of keypad
	for (;;) {
// Could wait for interrupt from P1IFG here
		keypadValue = KeypadRead();	// Read value from keypad
	}
}
//----------------------------------------------------------------------
// Set port 1 to default value between reads of keypad
// Columns outputs driven low, rows inputs with pullups
//----------------------------------------------------------------------
void KeypadInit (void)
{
	P1DIR = COLSMASK;			// Rows input, columns output
	P1OUT = ROWSMASK;
	P1REN = ROWSMASK;			//  Internal pullups on rows
}
//----------------------------------------------------------------------
// Read value from keypad
// Drive each column low in turn and read input from rows
//----------------------------------------------------------------------
int16_t	KeypadRead (void)
{
	uint8_t	column, row;		// Column and row being processed
	int16_t value;				// Value to be returned

	if ((P1IN & ROWSMASK) == ROWSMASK) {
		value = NOKEYPRESSED;	// No key pressed: don't scan
	} else {
		value = 0;
		switch ((~P1IN) & ROWSMASK) {	// Test for valid rows
			case 0x01:	row = 0;			break;
			case 0x02:	row = 1;			break;
			case 0x04:	row = 2;			break;
			case 0x08:	row = 3;			break;
			default:	value = KEYERROR;	break;
		}
		if (value != KEYERROR) {	// Row valid, scan columns
			P1DIR = ROWSMASK;		// Rows output, columns input
			P1OUT = COLSMASK;
			P1REN = COLSMASK;		//  Internal pullups on columns
// May need short delay here for inputs to stabilize
// Sequence of columns inverted so that array matches layout better
			switch ((~P1IN) & COLSMASK) {	// Test for valid rows
				case 0x10:	column = 2;			break;
				case 0x20:	column = 1;			break;
				case 0x40:	column = 0;			break;
//				case 0x80:	column = 3;			break;
				default:	value = KEYERROR;	break;
			}
			if (value != KEYERROR) {	// Column valid
				value = keyValue[row][column];
			}
			KeypadInit();				// Restore default settings
		}
	}
	return value;
}
