// press3.c - Timer_A for time between press and release of S2 = CCI0B
// LED4 lights while button down to show measurement in progress
// TI Experimenter's Board with F4619, default clocks
// J H Davies, 2007-06-07; IAR Kickstart version 3.42A
//----------------------------------------------------------------------
#include <io430xG46x.h>					// Specific device
#include <intrinsics.h>					// Intrinsic functions
#include <stdint.h>						// Integers of defined sizes
#include "LCDutils.h"					// SBLCDA4 utility functions
#define	LED		P5OUT_bit.P5OUT_1		// Pin for LED to show S2 down
void PortsInit (void);					// Set up i/o ports
//----------------------------------------------------------------------
void main (void)
{
	WDTCTL = WDTPW | WDTHOLD;			// Stop watchdog timer
	FLL_CTL0 = XCAP14PF;				// Configure load caps (14pF)
	PortsInit();						// Initialize ports
	LCDInit();							// Initialize SBLCDA4
	DisplayLine();						// Display ------- on LCD
// Capture falling edge of CCI0B, synchronized, interrupts enabled
	TACCTL0 = CM_2 | CCIS_1 | SCS | CAP | CCIE;
// Start timer: ACLK, no prescale, continuous mode, no ints, clear
	TACTL = TASSEL_1 | ID_0 | MC_2 | TACLR;
	for (;;) {							// Loop forever with interrupts
		__low_power_mode_3();			// Only ACLK continues to run
	}
}
//----------------------------------------------------------------------
// Interrupt service routine for TACCR0.CCIFG, called on capture
//----------------------------------------------------------------------
#pragma vector = TIMERA0_VECTOR
__interrupt void TIMERA0_ISR (void)		// Flag cleared automatically
{
	static uint16_t LastTime = 0;		// Last time captured

	if ((TACCTL0 & CM_2) != 0) {		// Falling edge, S2 pressed
		LastTime = TACCR0;				// Save time for next capture
		LED = 1;						// Light LED
	} else {							// Rising edge, S2 released
		DisplayUint(TACCR0 - LastTime);	// Display interval (counts)
		LED = 0;						// Extinguish LED
	}
	TACCTL0 ^= CM_3;					// Toggle polarity for next edge
}
//----------------------------------------------------------------------
// Initialize ports for board: outputs driven low by default
// Many of these will be overwritten by LCD initialization later
//----------------------------------------------------------------------
void PortsInit (void)
{
// Port 1: 0 = SW1, 1 = SW2, others used for Chipcon (not placed)
	P1OUT = 0;
	P1DIR = 0xFF & ~(BIT0|BIT1);		// P1.0,1 input, others output
	P1SEL = BIT1;						// P1.1 = S2 to Timer_A CCI0B
// Port 2: 1 = LED2, 2 = LED1, 3 = mic power, 4 = UCA0TXD, 5 = UCA0RXD
	P2OUT = 0;
	P2DIR = 0xFF & ~BIT5;				// P2.5 must be input
// Port 3: 0-3 = comms with F2013, 5 = buzzer (active low)
//	P3OUT = BIT1 | BIT2;				// F2013 might tie these high
//	P3DIR = 0xFF & ~BIT5;				// P3.5 must be high
	P3OUT = BIT1 | BIT2 | BIT5;			// F2013 might tie 1,2 high;
	P3DIR = 0xFF;						//   P3.5 high for buzzer off
// Port 4: 2-5 used for Chipcon (not placed)
	P4OUT = 0;
	P4DIR = 0xFF;
// Port 5: 1 = LED4, 2-4 = COM1-3 for LCD
	P5OUT = 0;
	P5DIR = 0xFF;
// Port 6: All except 6 used for opamp external components
	P6OUT = 0;
	P6DIR = 0xFF;
// Port 7: 4-7 = LCD segments
	P7OUT = 0;
	P7DIR = 0xFF;
// Port 8: LCD segments
	P8OUT = 0;
	P8DIR = 0xFF;
// Port 9: LCD segments
	P9OUT = 0;
	P9DIR = 0xFF;
// Port 10: 0-5 = LCD segments
	P10OUT = 0;
	P10DIR = 0xFF;
}
//----------------------------------------------------------------------
// Modifications to reconfigure clock for higher frequencies
//----------------------------------------------------------------------
// To reconfigure clock for 2.1 MHz
//	FLL_CTL0 = XCAP14PF;				// Configure load caps (14pF)
//	SCFQCTL = SCFQ_2M;					// DCO = 64 x ACLK = 2.1 MHz
// To reconfigure clock for 4.2 MHz
//	FLL_CTL0 = DCOPLUS | XCAP14PF;		// Config caps (14pF) and mult
//	SCFQCTL = SCFQ_2M;					// DCO = 64 x ACLK = 2.1 MHz
//	SCFI0 = FLLD_2 | FN_4;				// Double again, DCO range 4
