// quiz1.c - Light LED for 1s corresponding to first button pressed
// Interrupts on input high->low edges, LPM4 while waiting
// Timer_A to give interrupt after 1s delay, LPM3 while waiting
// Most action in main routine, minimum in ISRs
// Olimex 1121STK board, LED1,2 active low on P2.3,4
//   buttons B1, B2 active low on P2.1, P1.2
// J H Davies, 2006-12-19; IAR Kickstart version 3.41A
//----------------------------------------------------------------------
#include <io430x11x1.h>				// Specific device, new format
#include <intrinsics.h>				// Intrinsic functions
//----------------------------------------------------------------------
// Pins for LED and button
#define	LED1	P2OUT_bit.P2OUT_3
#define	LED2	P2OUT_bit.P2OUT_4
#define	B1		P2IN_bit.P2IN_1
#define	B2		P1IN_bit.P1IN_2
//----------------------------------------------------------------------
void main (void)
{
	WDTCTL = WDTPW|WDTHOLD;			// Stop watchdog timer
	P2OUT = BIT3|BIT4;				// Both LEDs off (active low!)
	P2DIR = BIT3|BIT4;				// Set pins with LEDs to output
	P2IES_bit.P2IES_1 = 1;			// Select high-to-low triggers
	P1IES_bit.P1IES_2 = 1;			//   for P2.1 and P1.2
	TACCR0 = 0x8000;				// Upper limit of count for TAR
	TACCTL0 = CCIE;					// Enable interrupts on Compare 0
	for (;;) {						// Loop forever
		while ((B1 == 0) || (B2 == 0)) {	// Wait for buttons
		}							// to be released (active low)
		do {
			P1IFG = 0;				// Clear any pending interrupts...
			P2IFG = 0;				// ...until none remain
		} while ((P1IFG != 0) || (P2IFG != 0));
		P2IE_bit.P2IE_1 = 1;		// Enable interrupts on edge
		P1IE_bit.P1IE_2 = 1;
		__low_power_mode_4();		// LPM4, all clocks stopped
// Wait for button to be pressed
		P2IE_bit.P2IE_1 = 0;		// Disable interrupts on edge
		P1IE_bit.P1IE_2 = 0;
		TACTL = MC_1|TASSEL_1|TACLR;	// Set up and start Timer A
// "Up to CCR0" mode, no clock division, clock from ACLK, clear timer
		__low_power_mode_3();		// LPM3, ACLK running for timer
// Wait for timer to expire
		TACTL_bit.TAMC = 0;			// Disable timer (mode 0)
		P2OUT = BIT3|BIT4;			// Both LEDs off (active low!)
	}
}
//----------------------------------------------------------------------
// Interrupt service routine for P2.1 (button 1)
// Acknowledge interrupt, light LED, return to active mode on exit
//----------------------------------------------------------------------
#pragma vector = PORT2_VECTOR
__interrupt void PORT2_ISR (void)
{
	P2IFG = 0;						// Clear interrupt flag(s)
	LED1 = 0;						// Light corresponding LED
	__low_power_mode_off_on_exit();	// Return to active mode on exit
}
//----------------------------------------------------------------------
// Interrupt service routine for P1.2 (button 2)
// Acknowledge interrupt, light LED, return to active mode on exit
// Should really check which pin caused interrupt but unnecessary here
//----------------------------------------------------------------------
#pragma vector = PORT1_VECTOR
__interrupt void PORT1_ISR (void)
{
	P1IFG = 0;						// Clear interrupt flag(s)
	LED2 = 0;						// Light corresponding LED
	__low_power_mode_off_on_exit();	// Return to active mode on exit
}
//----------------------------------------------------------------------
// Interrupt service routine for Timer A channel 0
// Returns to active mode after ISR
//----------------------------------------------------------------------
#pragma vector = TIMERA0_VECTOR
__interrupt void TA0_ISR (void)
{
	__low_power_mode_off_on_exit();	// Return to active mode on exit
}
