// shiftreg1.c - fill shift register with input from button
// Input from button -> shift register -> LED
// Poll free-running timer A with period of about 0.16s
// Timer clock is SMCLK divided by 2, continuous mode
// Olimex 1121STK, LED1 active low on P2.3, B1 active low on P2.1
// J H Davies, 2006-07-28; IAR Kickstart version 3.41A
//----------------------------------------------------------------------
#include <io430x11x1.h>				// Header file for this device
#include <stdint.h>

// Pins for LED and button, both active low
#define	LED1	P2OUT_bit.P2OUT_3
#define	BUT1	P2IN_bit.P2IN_1

void WaitTimer(void);				// Wait for timer to pace loop

void main (void)
{
	uint8_t ShiftReg;				// Shift register
// A bit field would be clearer for input and output

	WDTCTL = WDTPW | WDTHOLD;		// Stop watchdog timer
	LED1 = 1;						// Preload LED1 off (active low!)
	P2DIR_bit.P2DIR_3 = 1;			// Set pin with LED1 to output
	ShiftReg = 0xFF;				// Preload all 1s (active low)
	TACTL = MC_2|ID_1|TASSEL_2; 	// Set up Timer A
// Continous up mode, divide clock by 2, clock from SMCLK
	for (;;) {						// Paced loop forever
		WaitTimer();				// Wait for timer to start new loop
		LED1 = (ShiftReg & BIT0);	// Send oldest bit to LED
		ShiftReg >>= 1;				// Shift one place right
		if (BUT1 == 1) {			// Insert msb from state of button
			ShiftReg |= BIT7;
		} else {
			ShiftReg &= ~BIT7;
		}
	}
}
//----------------------------------------------------------------------
void WaitTimer(void)
{
	while (TACTL_bit.TAIFG == 0) {	// Wait for overflow
	}								//   doing nothing
	TACTL_bit.TAIFG = 0;			// Clear overflow flag
}
