// shiftreg1.c - fill shift register with input from button
// Input from button -> shift register -> LED
// Poll free-running timer A with period of about 0.16s
// Timer clock is SMCLK divided by 2, continuous mode
// Olimex 1121STK, LED1 active low on P2.3, B1 active low on P2.1
// J H Davies, 2006-07-28; IAR Kickstart version 3.41A
//----------------------------------------------------------------------
#include <io430x11x1.h>				// Header file for this device

// Pins for LED and button, both active low
#define	LED1	P2OUT_bit.P2OUT_3
#define	BUT1	P2IN_bit.P2IN_1

void WaitTimer(void);				// Wait for timer to pace loop

void main (void)
{
	union {
		unsigned char Byte;	
		struct {
			unsigned char lsb	: 1;
			unsigned char body	: 6;
			unsigned char msb	: 1;
		} Bits;
	} ShiftReg;						// Shift register as bits or byte

	WDTCTL = WDTPW | WDTHOLD;		// Stop watchdog timer
	LED1 = 1;						// Preload LED1 off (active low!)
	P2DIR_bit.P2DIR_3 = 1;			// Set pin with LED1 to output
	ShiftReg.Byte = 0xFF;			// Preload all 1s (active low)
	TACTL = MC_2|ID_1|TASSEL_2; 	// Set up Timer A
// Continous up mode, divide clock by 2, clock from SMCLK
	for (;;) {						// Paced loop forever
		WaitTimer();				// Wait for timer to start new loop
		LED1 = ShiftReg.Bits.lsb;	// Send oldest bit to LED
		ShiftReg.Byte >>= 1;		// Shift one place right
		ShiftReg.Bits.msb = BUT1;	// Insert msb from state of button
	}
}
//----------------------------------------------------------------------
void WaitTimer(void)
{
	while (TACTL_bit.TAIFG == 0) {	// Wait for overflow
	}								//   doing nothing
	TACTL_bit.TAIFG = 0;			// Clear overflow flag
}
