// testlcd3.c - test SBLCD4 display on TI Experimenter's Board
// Default clocks; ACLK from 32KHz crystal; minimal configuration
// Turns on all segments sequentially, then flashes all of them
// Flashes LEDs 1 and 2 on P2.2 and P2.1 to confirm operation of timer
// Timer clock 32KHz ACLK, no division, up mode,
//   period 0x1000 = 1/8 s, initially, raised to 0x4000 = 1/2s later
// J H Davies, 2006-12-02; IAR Kickstart version 3.41A
//----------------------------------------------------------------------
#include <io430xG46x.h>					// Specific device
#include <intrinsics.h>					// Intrinsic functions
#include <stdint.h>						// Integers of defined sizes
//----------------------------------------------------------------------
#define LCDMEMS	11						// LCD memories used (3-13)
// Pointer to LCD memory used: allows use of array LCDMem[]
uint8_t * const LCDMem = (uint8_t *) &LCDM3;	
//----------------------------------------------------------------------
// Function prototypes
void InitLCD (void);
//----------------------------------------------------------------------
void main (void)
{
	int i, j;							// Loop counters
	WDTCTL = WDTPW|WDTHOLD;				// Stop watchdog timer
	P2OUT = BIT2;					// LED1 (P2.2) on, LED2 (P2.1) off
	P2DIR = BIT1|BIT2;					// Set pins with LEDs to output
	InitLCD();							// Initialize SBLCDA4
	TACCR0 = 0x1000;					// Upper limit of count for TAR
	TACCTL0 = CCIE;						// Enable Compare 0 interrupts
	TACTL = MC_1|TASSEL_1|TACLR;		// Set up and start Timer A
// "Up to CCR0" mode, no clock division, clock from ACLK, clear timer
	for (i = 0; i < LCDMEMS; ++i) {		// Count through LCD memories
		for (j = 0; j < 8; ++j) {		// Count through bits
			LCDMem[i] = (LCDMem[i] << 1) | BIT0;	
			__low_power_mode_3();		// LPM3 provides delay
		}
		__low_power_mode_3();			// extra delays after character
		__low_power_mode_3();
	}
	TACCR0 = 0x4000;					// Slow down interrupts
	for (;;) {							// Loop forever
		LCDACTL_bit.LCDSON ^= 1;		// Toggle segments on and off
		__low_power_mode_3();			// LPM3 provides delay
	}
}
//----------------------------------------------------------------------
// Initialize SBLCDA4
//----------------------------------------------------------------------
void InitLCD (void)
{
	int i;
	for(i = 0; i < LCDMEMS; ++i) {		// Clear LCD memory used
		LCDMem[i] = 0;
	}
	P5SEL = BIT4 | BIT3 | BIT2;			// Select COM[3:1] function
	LCDAPCTL0 = LCDS4|LCDS8|LCDS12|LCDS16|LCDS20|LCDS24;	
								// Enable LCD segs 4-27 (4-25 used)
	LCDAVCTL0 = 0;				// No charge pump, everything internal
	LCDACTL = LCDFREQ_128 | LCD4MUX | LCDSON | LCDON;	
								// ACLK/128, 4mux, segments on, LCD_A on
}
//----------------------------------------------------------------------
// Interrupt service routine for Timer A channel 0
// Processor returns to "calling routine" in active mode
//----------------------------------------------------------------------
#pragma vector = TIMERA0_VECTOR
__interrupt void TIMERA0_ISR (void)
{
	P2OUT ^= BIT1|BIT2;					// Toggle LEDs
	__low_power_mode_off_on_exit();		// Restore Active Mode on return
}
